/*
 * @(#)datimtxt.java	1.00 2002-02-28 methodica herger & krasznai
 *
 */

// import java.util.Calendar;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.lang.Error;
import java.net.URL;

/*
 PURPOSE:<br>
 Server class providing a string that shows the current date
 and / or time.

 @author 	Hans E. Herger
 @version 1.0, 23 December 1998

 USAGE:<br>
 Usually, the DATIMTXT.CLASS is indirectly called by the
 TICKER.CLASS applet as a TICKER applet text server.
 To activate it, insert the parameter as shown below into
 your HTML &lt;APPLET&gt; .. &lt;/APPLET&gt; code:<br><pre>
    &lt;PARAM NAME="TextServer" VALUE="datimtxt"&gt;</pre><br>
 This applet accepts one additional parameter:<br><pre>
    &gt;PARAM NAME="DTFormat" VALUE="pattern"&gt;</pre><br>
 Where &quote;pattern&quote; is a string made up from the
 following characters:<br><pre>
    G        era designator          (Text)              AD
    y        year                    (Number)            1996
    M        month in year           (Text & Number)     July & 07
    d        day in month            (Number)            10
    h        hour in am/pm (1~12)    (Number)            12
    H        hour in day (0~23)      (Number)            0
    m        minute in hour          (Number)            30
    s        second in minute        (Number)            55
    S        millisecond             (Number)            978
    E        day in week             (Text)              Tuesday
    D        day in year             (Number)            189
    F        day of week in month    (Number)            2 (2nd Wed in July)
    w        week in year            (Number)            27
    W        week in month           (Number)            2
    a        am/pm marker            (Text)              PM
    k        hour in day (1~24)      (Number)            24
    K        hour in am/pm (0~11)    (Number)            0
    z        time zone               (Text)              Pacific Standard Time
    '        escape for text         (Delimiter)
    ''       single quote            (Literal)           '
 All other characters (e.g. hyphen, period, slant, etc) are
 taken literally.<br>
 Examples:<br><pre>
    &gt;PARAM NAME="DATIMTXT_1" VALUE="yyyy-MM-dd"&gt;
    &gt;PARAM NAME="DATIMTXT_1" VALUE="hh:mm a"&gt;
    &gt;PARAM NAME="DATIMTXT_1" VALUE="d/M/yy (EEE) HH:mm:ss:SSS (zzz)"&gt;</pre>

 NOTES:<br>
 TICKER applet text server classes must extend the TXTSRVR class.
 This class is an example of a specialized text server working
 together with the TICKER.CLASS applet.

 This code is freeware.<br>
 You are free to use and modify it, as long as due credit is given,
 and the resulting source code is made available to the public too.
 If you use this code, drop us a line ("mail@methodica.ch"), so
 we know, how many people are using it, and what an impact
 modifications to it will have.

 The code is given to the public domain on an "as is" basis.
 This means, that neither the author, nor methodica can be made
 responsible, under any circumstances, for damages caused by this
 applet, or malfunctions of this applet. Since the source code
 is in the public domain, you might convince yourself of the fact,
 that it doesn't contain offending or destructive code.

 2002-02-28, Hans E. Herger &lt;hans.herger@methodica.ch&gt;

*/


public class datimtxt extends txtsrvr
{
   /**
    * Data declarations
    */
   static final String defFormat   = "EEE, yyyy-MM-dd, HH:mm:ss (zzz)";
   protected String format         = defFormat;


   /**
    * Standard class constructor.
    * @see txtsrvr.java
    */
   public void txtsrvr()
   {
   } // end of constructor


   /**
    * Initializes the class.
    * @param UTL A unique resource locator (inherited from txtsrvr.java, not used here).
    * @param sParam A string parameter (inherited from txtsrvr.java, not used here).
    * @return a boolean, true if initialization was successful, false else
    */
	public boolean initText(URL uParam, String sParam, long lParam, int iParam)
	{

	   String s = getParameter("DATIMTXT_1");
	   if (s.length() > 0) {  format = s; }
	   return true;

	} // end of method initText


   // Exits the class.
   //--------------------------------------------------------------------------

   /**
    * Terminate the object. Called immediately before destroying it.
    */
	public void exitText()
	{
	} // end of method exitText


    // Returns the text to be displayed.
    //--------------------------------------------------------------------------

    public String getText()
    {
      SimpleDateFormat formatter = new SimpleDateFormat (format);
      Date rightNow = new Date();
      String msg = formatter.format(rightNow);
      return msg;
    } // end of method getText

}  // end of class datimtxt

