/*
 * @(#)itimtxt.java	1.00 1998-12-23 methodica herger & krasznai
 *
 */

import java.util.Date;
// import java.util.Calendar;
import java.net.URL;
import java.lang.Error;
import java.lang.Math;

/**
 * PURPOSE:<br>
 * Server class providing a string showing the universal internet
 * time as proposed by MIT and Swatch.
 * The internet time is the same around the globe. So no time
 * zones and no geographical borders apply. It is measured in 
 * "Swatch Beats" (SB). One SB is the equivalend of 86.4 seconds, 
 * thus 1000 SB giving one day. Biel Mean Time (Biel is the city
 * where SMH, the makers of Swatch watches, reside) is the universal
 * reference for internet time. BMT corresponds to Central European
 * Wintertime (e.g. GMT + 0100). Internet time begins at midnight BMT
 * (@000 SB). Noon (12:00) is the equivalent for @500 Swatch Beats.
 *
 * @author 	Hans E. Herger
 * @version 1.0, 23 December 1998
 *
 * USAGE:<br>
 * Usually, the ITIMTXT.CLASS is indirectly called by the
 * TICKER.CLASS applet as a TICKER applet text server.
 * To activate it, insert the parameter as shown below into
 * your HTML &lt;APPLET&gt; .. &lt;/APPLET&gt; code:
 * &lt;PARAM NAME="TextServer" VALUE="itimtxt"&gt;
 *
 * NOTES:<br>
 * TICKER applet text server classes must extend the TXTSRVR class.
 * This class is an example of a specialized text server working
 * together with the TICKER.CLASS applet.
 *
 * This code is freeware.<br>
 * You are free to use and modify it, as long as due credit is given,
 * and the resulting source code is made available to the public too.
 * If you use this code, drop us a line ("mail@methodica.ch"), so
 * we know, how many people are using it, and what an impact
 * modifications to it will have.
 *
 * The code is given to the public domain on an "as is" basis.
 * This means, that neither the author, nor methodica can be made
 * responsible, under any circumstances, for damages caused by this
 * applet, or malfunctions of this applet. Since the source code
 * is in the public domain, you might convince yourself of the fact,
 * that it doesn't contain offending or destructive code.
 *
 * 1998-12-23, Hans E. Herger &lt;hans.herger@methodica.ch&gt;
 *
 */
public class itimtxt extends txtsrvr
{
    private Date    ToDate;
//    private Calendar ToDate;
    private String  sError = null;
    private boolean ShowDecimals = false;
    private String  ds = "Wed, 23 Dec 1998 ";

    /**
     * Standard class constructor.
     * @see txtsrvr.java
     */
    public void txtsrvr()
    {
    } // end of constructor

    /**
     * Initializes the class.
     * @param URL A unique resource locator (inherited from txtsrvr.java, not used here).
     * @param sParam A string parameter (inherited from txtsrvr.java, not used here).
     * @return a boolean, true if initialization was successful, false else
     */
	public boolean initText(URL uParam, String sParam, long lParam, int iParam)
	{
	    String s = getParameter("ITIMTXT_1");
	    if (s.equalsIgnoreCase("SHOWDECIMALS"))
	    {
	        ShowDecimals = true;
	    }
//       ToDate = Calendar.getInstance();
	    return true;
	} // end of method initText

    // Exits the class.
    //--------------------------------------------------------------------------

    /**
     * Terminate the object. Called immediately before destroying it.
     */
	public void exitText()
	{
	} // end of method exitText


    /**
     * Gets and adjusts the time zone offset from a given Date object.
     * The time zone offset (in minutes) is returned incorrectly by the
     * Date object, so we are adjusting it here.
     * @param aDate A Date object.
     * @return the adjusted time zone offset in minutes.
     * @see Java.util.Date#getTimezoneOffset
     */
//	private static long adjustedTimezoneOffset(Calendar aCal)
	private static long adjustedTimezoneOffset(Date aDate)
	{
		long offset = aDate.getTimezoneOffset();
//      long offset =
//               (aCal.get(aCal.ZONE_OFFSET) / 1000) +
//               (aCal.get(aCal.DST_OFFSET) / 1000);

//      return offset;
		return  (long) (( offset + ((offset >= 0) ? +15 : -15)) / 30) * 30 ;
	}

    /**
     * Gets the current time, standardizes it to correspond to GMT,
     * calculates Swatch beats and returns a formatted string
     * representing the internet time.
     * @return the formatted date / time string.
     * @see #adjustedTimezoneOffset
     */
    public String getText()
    {
        double beats;
        long secs,t;
        String msg;

        if (sError != null) return sError;

        ToDate = new Date ();
//        ToDate = Calendar.getInstance();

//        secs = (int)( ToDate.get(ToDate.SECOND) +
//                     (ToDate.get(ToDate.MINUTE) * 60) +
//                     ((ToDate.get(ToDate.AM_PM) * 12 )+ ToDate.get(ToDate.HOUR) * 3600) +
//                     adjustedTimezoneOffset(ToDate));

        secs = (int)( ToDate.getSeconds()+
                      (ToDate.getMinutes() * 60) +
                      (ToDate.getHours() * 3600) +
                      (adjustedTimezoneOffset(ToDate) * 60) + 90000);
        beats = (secs / 86.4) % 1000;

        msg = new String("@");
        if (ShowDecimals)
        {
            msg += beats;
        }
        else
        {
            msg += (int)beats;
        }

        return msg;
    } // end of method getText

}
