/*
 * @(#)y2kdtxt.java	1.00 1998-06-10 methodica herger & krasznai
 *
 */

import java.util.Date;
import java.net.URL;
import java.lang.Error;

/**
 * PURPOSE:<br>
 * Server class providing a string showing the calculate
 * difference between two dates.
 * If either or both of the dates is specified by parameters,
 * this date is used, the date today ist assumed else.
 * If none of the date parameters is specified, the server
 * calculates the difference between today and January 1st,
 * 2000 at 00:00:00 local time.
 *
 *
 * @author 	Hans E. Herger
 * @version 1.0, 23 June 1998
 *
 * USAGE:<br>
 * Usually, the Y2KDTXT.CLASS is indirectly called by the
 * TICKER.CLASS applet as a TICKER applet text server.
 * To activate it, insert the parameter as shown below into
 * your HTML &lt;APPLET&gt; .. &lt;/APPLET&gt; code:
 * &lt;PARAM NAME="TextServer" VALUE="y2kdtxt"&gt;
 *
 * The Y2KDTXT_1 parameter specifies the FROM-DATE. E.g:<br>
 * &lt;PARAM NAME="Y2KDTXT_1" VALUE="1970 Jun 11 02:00:00"&gt;<br>
 * The Y2KDTXT_2 parameter specifies the TO-DATE. E.g:<br>
 * &lt;PARAM NAME="Y2KDTXT_2" VALUE="1998 Nov 24"&gt;
 *
 * NOTES:<br>
 * TICKER applet text server classes must extend the TXTSRVR class.
 * This class is an example of a specialized text server working
 * together with the TICKER.CLASS applet.
 *
 * This code is freeware.<br>
 * You are free to use and modify it, as
 * long as due credit is given, and the resulting source code is
 * made available to the public too.
 * If you use this code, drop us a line ("mail@Tmethodica.ch"), so
 * that we know, how many people are using it, and what an impact
 * modifications to it will have.
 *
 * The code is given to the public domain on an "as is" basis.
 * This means, that neither the author, nor methodica can be made
 * responsible, under any circumstances, for damages caused by this
 * applet, or malfunctions of this applet. Since the source code
 * is in the public domain, you might convince yourself of the fact,
 * that it doesn't contain offending or destructive code.
 *
 * 1998-06-10, Hans E. Herger &lt;hans.herger@methodica.ch&gt;
 *
 */
public class y2kdtxt extends txtsrvr
{
    static  final int SecondsPerMinute = 60;
    static  final int MinutesPerHour = 60;
    static  final int MinutesPerDay = 1440;
    static  final int MinutesPerWeek = 10080;
    static  final int BaseYear = 1900;

    static final String empty   = "";
    static final String zero    = "0";
    static final String period  = ":";
    static final String hyphen  = "-";
    static final String blank   = " ";

    private Date    ToDate, FromDate;
    private Date    FixToDate = null;
    private Date    FixFromDate = null;
	private boolean bY2KDefault = false;
    private int     yToDate, yFromDate;
	private boolean lyToDate, lyFromDate;
	private long    deltaToDate, deltaFromDate;
	private String  sError = null;
    private boolean isMSIE401 = false;
	private long    deltaTZO = 0;

    /**
     * Standard class constructor.
     * @see txtsrvr.java
     */
    public void txtsrvr()
    {
    } // end of constructor

    /**
     * Creates a string stating date parameter error (y2kdtxt_1, y2kdtxt_2).
     * @param name The name of the invalid parameter.
     * @param value The value of the invalid parameter.
     * @return the string created.
     */
    private static String DateParameterError(String name, String value)
    {
        return new String("ERROR: PARAM NAME=" + name +
                    " VALUE=" + value +
	                " (probably you specified a date before 1970-01-01)" );
	}

    /**
     * Initializes the class.
     * @param UTL A unique resource locator (inherited from txtsrvr.java, not used here).
     * @param sParam A string parameter (inherited from txtsrvr.java, not used here).
     * @return a boolean, true if initialization was successful, false else
     */
	public boolean initText(URL uParam, String sParam, long lParam, int iParam)
	{

	    bY2KDefault = false;
	    String s = getParameter("Y2KDTXT_1");
	    if (s.length() > 0)
	    {
	        try
	        {
	            FixFromDate = new Date(s);
	        }
	        catch (IllegalArgumentException e)
	        {
	            sError = DateParameterError("Y2KDTXT_1 (=FromDate)", s);
	        }
	    }
	    s = getParameter("Y2KDTXT_2");
	    if (s.length() > 0)
	    {
	        try
	        {
	            FixToDate = new Date(s);
	        }
	        catch (IllegalArgumentException e)
	        {
	            sError = DateParameterError("Y2KDTXT_2 (=ToDate)", s);
	        }
	    }

	    // isMSIE401 = Is it the MS Internet Explorer 4.01 we use?

	    return true;
	} // end of method initText

    // Exits the class.
    //--------------------------------------------------------------------------

    /**
     * Terminate the object. Called immediately before destroying it.
     */
	public void exitText()
	{
	} // end of method exitText


    // Returns the text to be displayed.
    //--------------------------------------------------------------------------

    /**
     * Creates a specific date string from a Date object. Form "yyyy-mm-dd".
     * @param dt A Date object.
     * @return the string created.
     * @see Java.util.Date
     */
    private static String DateString(Date dt)
    {
        int y = (dt.getYear() + BaseYear);
        byte m = (byte) (dt.getMonth() + 1);
        byte d = (byte) dt.getDate();
        return new String(
            y + hyphen +
            ((m<10) ? zero : empty) + m + hyphen +
            ((d<10) ? zero : empty) + d );
    }

    /**
     * Creates a time string from a Date object. Form "HH:MM:SS".
     * @param dt A Date object.
     * @return the string created.
     * @see Java.util.Date
     */
    private static String TimeString(Date dt)
    {
        byte H = (byte) dt.getHours();
        byte M = (byte) dt.getMinutes();
        byte S = (byte) dt.getSeconds();
        return new String(
            ((H<10) ? zero : empty) + H + period +
            ((M<10) ? zero : empty) + M + period +
            ((S<10) ? zero : empty) + S );
    }

    /**
     * Creates a string from the time zone offset. Format "(UCT-HHMM)"
     * @param dt A Date object.
     * @return the string created.
     * @see #adjustedTimezoneOffset
     * @see #TimeString
     */
    private static String UCTTimeString(Date dt)
    {
        byte dM = (byte) -adjustedTimezoneOffset(dt);
        if (dM == 0)
        {
            return TimeString(dt) + " (UCT)";
        }
        else
        {
            String sign = ((dM != 0) ? ((dM<0) ? "-" : "+" ) : "");
            byte dH = (byte) Math.abs(dM / 60);
            dM = (byte) ((dM % 60 + 1) / 10 * 10);
            dH += dM / 60; dM %= 60;

            return  TimeString(dt) + " (UCT" + sign +
                    ((dH<10) ? zero : empty) + dH +
                    ((dM<10) ? zero : empty) + dM + ")";
        }
    }

    /**
     * Creates a normalized date, time and time zone offset string.
     * @param dt A Date object.
     * @param bUCT A boolean value; true if time zone offset to be included.
     * @return the string created.
     * @see #DateString
     * @see #TimeString
     * @see #UCTTimeString
     */
    private static String NormalizeDateTime(Date dt, boolean bUCT)
    {
        return DateString(dt) + blank +
            ((bUCT) ? UCTTimeString(dt) : TimeString(dt));
    }

    /**
     * Gets and adjusts the time zone offset from a given Date object.
     * This value is not returned correctly by the Date object, so
     * we try to adjust it here.
     * @param curDate A Date object.
     * @return the adjusted time zone offset in minutes.
     * @see Java.util.Date#getTimezoneOffset
     */
	private static long adjustedTimezoneOffset(Date curDate)
	{
		long offset = curDate.getTimezoneOffset();
		return  (long) (( offset + ((offset >= 0) ? +15 : -15)) / 30) * 30 ;
	}

    /**
     * Calculates the date and time difference formats a string showing
     * the result.
     * @return the formatted date / time string.
     * @see #adjustedTimezoneOffset
     */
    public String getText()
    {
        // Note: Years are 1900-relative, monthes are 0-relative,
        //       days are 1-relative.
        int c,m;
        long weks,days,hors,mins,secs;
        String msg, t1, t2;

        if (sError != null) return sError;

        if ((FixFromDate == null) && (FixToDate == null))
        {
            // assume 2000-01-01 00:00:00 when no limit specified
            FixToDate = new Date(100,0,1,0,0,0);
            bY2KDefault = true;
        }

        if (FixToDate == null)      { ToDate = new Date (); }
        else ToDate = FixToDate;

        if (FixFromDate == null)    {FromDate = new Date (); }
        else FromDate = FixFromDate;

        if (ToDate.before(FromDate))
        {
            // swap if upper limit < lower limit
            Date td = FromDate; FromDate = ToDate; ToDate = td;
            td = FixFromDate; FixFromDate = FixToDate; FixToDate = td;
        }

        boolean bOnlyOneLimit = ((FixFromDate == null) || (FixToDate == null));

        // Adjust, for not all java VM are working the same way. Microsoft IE 4.01
        // does init a Date differently when year, month, date explicitely are
        // specified, from the default (today) date ?!

        long deltaTZO; // delta in seconds
        if (isMSIE401)
        {
            deltaTZO = (!bOnlyOneLimit) ?
                             0 :
                             (( adjustedTimezoneOffset(ToDate) -
                                adjustedTimezoneOffset(FromDate)
                              ) * SecondsPerMinute);
        }
        else deltaTZO = 0;

        secs = ((ToDate.getTime() - FromDate.getTime()) / 1000) - deltaTZO;
        mins = secs / SecondsPerMinute;
        secs %= SecondsPerMinute;

        msg = new String
                (
                    bOnlyOneLimit ?
                    (
                        (   (FixFromDate != null) ?
                            "Seit " + NormalizeDateTime(FromDate, false) :
                            "Bis "  + NormalizeDateTime(ToDate, false)
                        )
                    ) :
                    (
                        NormalizeDateTime(FromDate, true) +
                        " .. " + NormalizeDateTime(ToDate, true)
                    )
                );
        msg += " -> ";

//        if (bY2KDefault)
//        {
//        	weks = (int)(mins/MinutesPerWeek);  mins %= MinutesPerWeek;
//        }
//        else
            weks = 0;
    	days = (int)(mins/MinutesPerDay);   mins %= MinutesPerDay;
    	hors = (int)(mins/MinutesPerHour);  mins %= MinutesPerHour;

        if (weks>0)
        {
            msg += (int)weks + " Woche";
            if (weks!=1) {msg+="n";}
            msg += ", ";
        }
        if (days > 0)
        {
      	    msg += (int)days + " Tag";
      	    if (days!=1) {msg+="e";}
      	    msg += ", ";
  	    }
  	    if (hors > 0)
  	    {
      	    if (hors < 10) {msg += zero;}
      	    msg += (int)hors;
      	    msg += ":";
      	    if (mins < 10) {msg += zero;}
      	    msg += (int)mins;
      	    msg += " Stunde";
      	    if (hors!=1) {msg+="n";}
      	}
      	else
      	{
      	    if (mins < 10) {msg += zero;}
      	    msg += (int)mins;
      	    msg += " Minute";
      	    if (mins!=1) {msg+="n";}
      	}

        return msg;
    } // end of method getText

}
